

from staq.stack import Stack, StackFrame, StackCell 
from staq.function import Function, FunctionArgument
from staq.archs.architecture import Architecture, Register
from typing import List
from cliify import commandParser, command

import re

@commandParser
class X86_cdecl(Architecture):
    def __init__(self, session = None, stack = None):
        super().__init__('cdecl', session=session, stack=stack)
        self.endian = 'little'


        self.registers = {
            'eax': Register('eax', 4, 'Return Value'),
            'ecx': Register('ecx', 4, 'Arg 1'),
            'edx': Register('edx', 4, 'Arg 2'),
            'ebx': Register('ebx', 4, 'Arg 3'),
            'esp': Register('esp', 4, 'Stack Pointer'),
            'ebp': Register('ebp', 4, 'Base Pointer'),
            'esi': Register('esi', 4, 'Source Index'),
            'edi': Register('edi', 4, 'Destination Index'),
            'eip': Register('eip', 4, 'Instruction Pointer')
        }

        self.setReg('esp', hex(self.stack.pointer))
        self.setReg('eip', '???')
        self.setReg('ebp', hex(self.stack.baseAddress))

    def clear(self):

        for key in self.registers:
            self.registers[key].value = None
        
        self.setReg('esp', hex(self.stack.pointer))
        self.setReg('eip', '???')
        self.setReg('ebp', hex(self.stack.baseAddress))



    def setInstructionPointer(self, val):

        self.setReg("eip", val)

    def setFramePointer(self, val):
        self.setReg("ebp", val)

    def setReturnValue(self, val):
        self.setReg("eax", val)
    
    
    


        


import sys
import os

from staq.stackSession import StackSession



def test_session_cmd():

    session = StackSession()

    completions = session.getCompletions("")
    assert "pop" in completions

    

# def test_session():

#     session = StackSession()

#     session.parseCommand("int i =42")

#     wordCount = len(session.stack.currentFrame.cells[0].words)
#     wordValue = session.stack.currentFrame.cells[0].words[0].value

#     assert wordCount == 1
#     assert wordValue == 42

# def test_html():
#     session = StackSession()
#     session.stack.baseAddress =0xffffff


#     session.parseCommand('int etc[64]')
#     session.parseCommand('call main(1,"Hello World")')
#     session.parseCommand('char buf[64]')
#     session.parseCommand('call strcpy(buf,argv[0])')
#     session.parseCommand('note 0xfeb3: note on local variable')


#     html = session.stack.toHtml()

#     with open("test.html", "w") as fh:
#         fh.write(html)

# def test_png():
#     session = StackSession()
#     session.stack.baseAddress =0xffffff
#     imagePath = 'test.png'
    
#     session.parseCommand('int etc[64]')
#     session.parseCommand('call main(1,"Hello World")')
#     session.parseCommand('char buf[64]')
#     session.parseCommand('call strcpy(buf,argv[0])')
#     session.parseCommand('note 0xfeb3: note on local variable')


#     session.stack.generatePng(imagePath)

#     assert os.path.exists(imagePath)


if __name__ == "__main__":
    test_session()




from staq.stack import Stack, StackFrame, StackCell 
from staq.function import Function, FunctionArgument
from staq.archs.architecture import Architecture, Register
from typing import List
from cliify import commandParser, command
from cliify.splitHelper import splitWithEscapes

import re

@commandParser
class Arm32(Architecture):
    def __init__(self, session = None, stack = None):
        super().__init__('arm', session=session, stack=stack)
        self.endian = 'little'



        self.registers = {
            'r0': Register('r0', 4, 'Return Value'),
            'r1': Register('r1', 4, 'Arg 1'),
            'r2': Register('r2', 4, 'Arg 2'),
            'r3': Register('r3', 4, 'Arg 3'),
            'r4': Register('r4', 4, 'Arg 4'),
            'r5': Register('r5', 4, 'Arg 5'),
            'r6': Register('r6', 4, 'Arg 6'),
            'sp': Register('sp', 4, 'Stack Pointer'),
            'fp': Register('fp', 4, 'Base Pointer'),
            'lr': Register('lr', 4, 'Link Register'),
            'pc': Register('pc', 4, 'Program Counter')
        }
        self.setReg('sp', hex(self.stack.pointer))
        self.setReg('pc', '???')
        self.setReg('fp', hex(self.stack.baseAddress))
        self.setReg('lr', '???')

    def clear(self):

        for key in self.registers:
            self.registers[key].value = None
        
        self.setReg('sp', hex(self.stack.pointer))
        self.setReg('pc', '???')
        self.setReg('fp', hex(self.stack.baseAddress))
        self.setReg('lr', '???')



    def setInstructionPointer(self, val):
        self.setReg("pc", val)

    def setFramePointer(self, val):
        return super().setFramePointer(val)
    
    def setReturnValue(self, val):
        self.setReg("r0", val)

    def updateRegs(self):
        self.setReg('sp', self.stack.pointer)

    
    
    


    @command(completions={'function': lambda self: self.session.functions.keys()})
    def call(self, function):
        """
        ARM32 implementation of the call command. Handles function calls according to ARM32 calling convention.
        Arguments are placed in r0-r3 registers first, with any additional arguments pushed onto the stack.
        """

        callLine = function.strip()
        regex = re.compile(r"(\S.*?)\((.*?)\)")
        match = regex.match(callLine)
        
        func = None
        
        if match:
            functionName = match.group(1)
            args = match.group(2)
        else:
            functionName = callLine
            args = ""
        
        if args.strip() == "":
            args = []
        else:
            argStr = match.group(2)
            args = splitWithEscapes(argStr, ',')
        
        # If the function does not exist, create it and assume all args are ints
        if functionName not in self.session.functions:
            newFunction = Function(functionName)
            
            for i in range(len(args)):
                newFunction.addArgument(FunctionArgument('int', f'arg{i+1}'))
            
            self.session.functions[functionName] = newFunction
        
        func = self.session.functions[functionName]
        
        # Save caller's lr register to keep track of return address
        self.setReg('lr', '<return_address>')
        
        # Handle arguments according to ARM32 calling convention
        # First 4 args go in r0-r3, rest on stack
        for i in range(len(args)):
            arg_value = args[i].strip()
            arg_label = f'arg{i+1}'
            
            # Use the argument name if available
            if i < len(func.args) and func.args[i].name:
                arg_label = func.args[i].name
            
            if i < 4:
                # First 4 args go in r0-r3
                self.setReg(f'r{i}', arg_value, note=arg_label)
            else:
                # Push remaining args on stack (in reverse order per ARM convention)
                # For ARM, we need to adjust the stack pointer
                cell = StackCell(arg_label, [arg_value])
                self.stack.push(cell)
        
        # Set up the return address on the stack
        currentFunction = self.session.getCurrentFunction()
        
        retCell = StackCell('ret', ['<???>'])
        if currentFunction:
            if isinstance(currentFunction, Function):
                retCell.setWords(f'<{currentFunction.name}+??>')
            else:
                retCell.setWords(f'<{currentFunction}+??>')
        
        self.stack.push(retCell)
        
        # Jump to the function (in ARM this would be a BL instruction)
        self.jmp(functionName)
        
        # Allocate local variables for the function
        if func:
            for local in func.locals:
                cell = self.session.tryParseLocalVar(local)
                if cell:
                    self.stack.push(cell)


from setuptools import find_packages, setup
import os
__version__ = "0.1.0"

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name='staq',
    version="0.2.6",
    description='Stack Visualization Tool',
    long_description=long_description,
    packages=find_packages(),
    package_dir={
        "staq": "staq"
    },
    include_package_data=True,
    package_data={
        'staq': ['templates/**/*.*', 'data/*.c'],
    },
    scripts=["staq/staq"], 
    install_requires = ["prompt_toolkit", "Sphinx", "termcolor", "ansi2html","pyyaml","jinja2", 'html2image', 'pillow'],
    entry_points = {
        'spinx.extension': [ 'stack = staq.stackDirective' ]
    }
)
